import { Type, ArgumentType } from './types/types';
import { Diagnostic } from 'vscode-languageserver';
import { SuffixTypeBuilder } from './suffixTypeNode';
/**
 * type result for a kerboscript expression
 */
export interface ITypeResultExpr<T extends Type> {
    /**
     * result type
     */
    type: T;
    /**
     * errors encountered during this expression type check
     */
    errors: Diagnostic[];
}
/**
 * type result for a kerboscript suffix
 */
export interface ITypeResultSuffix<T extends SuffixTypeBuilder = SuffixTypeBuilder> {
    /**
     * resolved cummulative suffix types
     */
    builder: T;
    /**
     * errors encounted during this suffix type check
     */
    errors: Diagnostic[];
}
/**
 * What is the kind of this type
 */
export declare const enum TypeKind {
    basic = 0,
    variadic = 1,
    suffix = 2,
    function = 3
}
/**
 * What is the kind of this call
 */
export declare const enum CallKind {
    get = 0,
    set = 1,
    call = 2,
    optionalCall = 3
}
/**
 * Construct a binary operation for a given type
 */
export interface BinaryConstructor {
    /**
     * What is the operator
     */
    operator: OperatorKind.and | OperatorKind.or | OperatorKind.plus | OperatorKind.subtract | OperatorKind.multiply | OperatorKind.divide | OperatorKind.power | OperatorKind.greaterThan | OperatorKind.lessThan | OperatorKind.greaterThanEqual | OperatorKind.lessThanEqual | OperatorKind.notEqual | OperatorKind.equal;
    /**
     * What is the return type of this operator
     */
    returnType: ArgumentType;
    /**
     * What is the other type of the operator
     */
    other: ArgumentType;
}
/**
 * Construct a unary operation for a given type
 */
export interface UnaryConstructor {
    /**
     * What is the operator
     */
    operator: OperatorKind.negate;
    /**
     * What is the return type of this operator
     */
    returnType: ArgumentType;
    /**
     * What is the other type of this operator
     */
    other: undefined;
}
/**
 * What operator kind is present
 */
export declare enum OperatorKind {
    not = 0,
    defined = 1,
    negate = 2,
    and = 3,
    or = 4,
    plus = 5,
    subtract = 6,
    multiply = 7,
    divide = 8,
    power = 9,
    greaterThan = 10,
    lessThan = 11,
    greaterThanEqual = 12,
    lessThanEqual = 13,
    notEqual = 14,
    equal = 15
}
